@@ -4,13 +4,16 @@ use warnings;
 use strict;
 use Carp;
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 sub engine {
-    my $class        = shift;
-    my %args         = @_;
-    my $type         = delete $args{type} or croak "type required";
-    my $engine_class = 'Search::OpenSearch::Engine::' . $type;
+    my $class = shift;
+    my %args  = @_;
+    my $type  = delete $args{type} or croak "type required";
+    my $engine_class
+        = $type =~ s/^\+//
+        ? $type
+        : 'Search::OpenSearch::Engine::' . $type;
     eval "use $engine_class";
     if ($@) {
         croak $@;
@@ -47,21 +50,15 @@ Search::OpenSearch - provide search results in OpenSearch format
     c           => 0,                   # return count stats only (no results)
     L           => 'field|low|high',    # limit results to inclusive range
     f           => 1,                   # include facets
+    r           => 1,                   # include results
     format      => 'XML',               # or JSON
+    b           => 'AND',               # or OR
  );
  print $response;
 
 =head1 DESCRIPTION
 
-This module is a work-in-progress. The following features are not yet implemented:
-
-=over
-
-=item facets
-
-=item highlighting
-
-=back
+This module is a work-in-progress. The API is subject to change.
 
 Search::OpenSearch is a framework for various backend engines
 to return results comforming to the OpenSearch API (http://opensearch.org/).
@@ -78,22 +75,6 @@ Peter Karman, C<< <karman at cpan.org> >>
 
 =head1 BUGS
 
-=over
-
-=item *
-
-Currently no tests. Major bug.
-
-=item *
-
-OpenSearch is not fully implemented in any Response format, particularly JSON.
-
-=item * 
-
-Facet support documented but non-existent.
-
-=back
-
 Please report any bugs or feature requests to C<bug-search-opensearch at rt.cpan.org>, or through
 the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Search-OpenSearch>.  I will be notified, and then you'll
 automatically be notified of progress on your bug as I make changes.
